/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.olaf.tools;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Component;
import java.awt.Container;
import java.awt.Window;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

public class LayoutTools {
  public static void constrains(Container container, Component component,
                               int x, int y, int w, int h, int f, int a,
                               double wx, double wy,
                               int t, int l, int b, int r) {
    GridBagConstraints gbc = null;

    gbc = new GridBagConstraints();
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = w;
    gbc.gridheight = h;
    gbc.fill = f;
    gbc.anchor = a;
    gbc.weightx = wx;
    gbc.weighty = wy;
    if (t+b+l+r > 0) {
      gbc.insets = new Insets(t, l, b, r);
    }

    ((GridBagLayout) container.getLayout()).setConstraints(component, gbc);
    container.add(component);
  }

  public static void centerOnScreen (Window window) {
    int       x         = 0;
    int       y         = 0;
    Dimension screenSize = null;
    Dimension windowSize = null;

    screenSize = window.getToolkit().getScreenSize();
    windowSize = window.getSize();

    x = (int) ((screenSize.getWidth() / 2) - (windowSize.getWidth() / 2));
    y = (int) ((screenSize.getHeight() / 2) - (windowSize.getHeight() / 2));

    window.setLocation(x, y);
  }

  public static void centerOnWindow(Window window, Window parent) {
    int       x          = 0;
    int       y          = 0;
    Point     parentLocation = null;
    Dimension parentSize = null;
    Dimension windowSize = null;

    parentSize = parent.getSize();
    parentLocation = parent.getLocation();
    windowSize = window.getSize();

    x = (int) (parentLocation.getX() + (parentSize.getWidth() / 2) - (windowSize.getWidth() / 2));
    y = (int) (parentLocation.getY() + (parentSize.getHeight() / 2) - (windowSize.getHeight() / 2));

    window.setLocation(x, y);
  }
}
