/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.olaf.widgets;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.SwingConstants;
import javax.swing.ImageIcon;
import javax.swing.Action;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JToggleButton;
import javax.swing.BorderFactory;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

public class DropDownButton extends JPanel implements MouseListener, PopupMenuListener {
  private JButton       button               = null;
  private JPopupMenu    menu                 = null;
  private JToggleButton select               = null;
  private boolean       setDefaultEnabled    = true;
  private boolean       removeDefaultEnabled = true;
  private boolean       menuIsVisible        = false;

  public DropDownButton() {
    ImageIcon  icon  = null;

    setPreferredSize(new Dimension(34, 23));
    setMinimumSize(new Dimension(34, 23));
    setMaximumSize(new Dimension(34, 23));

    setLayout(new BorderLayout());
    button = new JButton();
    button.setFocusPainted(false);
    button.setPreferredSize(new Dimension(20, 20));
    button.setMinimumSize(new Dimension(20, 20));
    button.addMouseListener(this);
    add(button,BorderLayout.CENTER);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/DropDownButtonArrow.png"));
    select = new JToggleButton();
    select.setFocusPainted(false);
    select.setIcon(icon);
    select.setPreferredSize(new Dimension(10, 18));
    select.setMinimumSize(new Dimension(10, 18));
    select.addMouseListener(this);
    add(select,BorderLayout.EAST);
    menu = new JPopupMenu();
    menu.addPopupMenuListener(this);
  }

  public void mouseClicked(MouseEvent e) {
    if(!menuIsVisible) {
      select.setSelected(false);
    }
  }

  public void mouseEntered(MouseEvent e) {
    Object source = null;

    source = e.getSource();
    if(source == button) {
        select.getModel().setRollover(true);
    } else {
        button.getModel().setRollover(true);
    }
  }

  public void mouseExited(MouseEvent e) {
    Object source = null;

    source = e.getSource();
    if(source == button) {
        select.getModel().setRollover(false);
    } else {
        button.getModel().setRollover(false);
    }
  }

  public void mousePressed(MouseEvent e) {
    Object source = null;

    source = e.getSource();
    if(source == select) {
        if ((!select.isSelected()) && isEnabled() && (e.getClickCount() == 1)) {
          menu.show(button, 0, button.getSize().height);
        }
    } else {
        select.getModel().setPressed(true);
    }
  }

  public void mouseReleased(MouseEvent e) {
    Object source = null;

    source = e.getSource();
    if(source == button) {
        select.getModel().setPressed(false);
    }
  }

  public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
    menuIsVisible = true;
    select.setSelected(true);
  }

  public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
    menuIsVisible = false;
    select.setSelected(false);
  }

  public void popupMenuCanceled(PopupMenuEvent e) {
  }

  public void setEnabled(boolean enabled) {
    super.setEnabled(enabled);
    button.setEnabled(enabled);
    select.setEnabled(enabled);
  }

  public void setText(String text) {
    button.setText(text);
  }

  public JButton getButton() {
    return button;
  }

  public void setDefaultAction(Action action) {
    setDefaultEnabled = false;
    removeDefaultEnabled = false;
    button.setAction(action);
  }

  public void addAction(Action action) {
    if(setDefaultEnabled) {
      setDefaultEnabled = false;
      button.setAction(action);
    }
    menu.add(action);
  }

  public void insertAction(Action action, int i) {
    if(setDefaultEnabled && (i == 0)) {
      setDefaultEnabled = false;
      button.setAction(action);
    }
    menu.insert(action, i);
  }

  public void setIcon(ImageIcon icon) {
    button.setIcon(icon);
  }

  public void setRolloverIcon(ImageIcon icon) {
    button.setRolloverIcon(icon);
  }

  public void setDisabledIcon(ImageIcon icon) {
    button.setDisabledIcon(icon);
  }

  public Action removeAction(int i) {
    Action action = null;

    action = ((JMenuItem)menu.getComponent(i)).getAction();
    menu.remove(i);
    if((i == 0) && (removeDefaultEnabled)) {
      button.setAction(((JMenuItem)menu.getComponent(0)).getAction());
    }

    return action;
  }

  public void setToolTipText(String toolTip) {
    button.setToolTipText(toolTip);
  }

  public static void main(String[] args) {
    DropDownButton ddb = new DropDownButton();

    Action a = new javax.swing.AbstractAction() {
      public void actionPerformed(java.awt.event.ActionEvent e) {
        System.out.println("action 1");
    }};
//    ddb.setDefaultAction(a);
    ddb.setText("Go Fish");

    a = new javax.swing.AbstractAction("item 1") {
      public void actionPerformed(java.awt.event.ActionEvent e) {
        System.out.println("action 2");
    }};
    ddb.addAction(a);
    a = new javax.swing.AbstractAction("item 2") {
      public void actionPerformed(java.awt.event.ActionEvent e) {
        System.out.println("action 3");
    }};
    ddb.addAction(a);
    a = new javax.swing.AbstractAction("item 3") {
      public void actionPerformed(java.awt.event.ActionEvent e) {
        System.out.println("action 4");
    }};
    ddb.addAction(a);
    ddb.removeAction(0);


    javax.swing.JToolBar jtb = new javax.swing.JToolBar();
    ddb.setEnabled(false);
    jtb.add(ddb);
    //show it
    javax.swing.JFrame frame = new javax.swing.JFrame("Snazzy drop down button");
    frame.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
    JPanel jp = new JPanel();
    jp.add(jtb);
    frame.setContentPane(jp);
    frame.pack();
    frame.setVisible(true);
    ddb.setEnabled(true);
  }
}
