/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.placide;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.ImageIcon;

import net.jourand.olaf.tools.LayoutTools;
import net.jourand.placide.beans.CommuneSearchBean;
import net.jourand.placide.io.PlacesLoader;
import net.jourand.placide.places.Places;

public class Launcher extends WindowAdapter {
    private CommuneSearchBean csb = null;
    private JFrame frame = null;

    public Launcher() {
        Places       places = null;
        ImageIcon    icon   = null;

        try {
            UIManager.setLookAndFeel("com.jgoodies.plaf.windows.ExtWindowsLookAndFeel");
            csb = new CommuneSearchBean();
            places = PlacesLoader.load();
            csb.setRegions(places.getRegions());
            csb.setDepartements(places.getDepartements());
            csb.setArrondissements(places.getArrondissements());
            csb.setCantons(places.getCantons());
            csb.setCommunes(places.getCommunes());
            csb.setHistories(places.getHistories());
            csb.setSearchables(places.getSearchables());
        } catch (Exception e) {
            e.printStackTrace();
        }

        frame = new JFrame("Placide");
        icon = new ImageIcon(getClass().getClassLoader().getResource("images/placide16.png"));
        frame.setIconImage(icon.getImage());
        frame.getContentPane().add(csb.getPanel());
        frame.addWindowListener(this);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        csb.setOwner(frame);
        frame.pack();
        LayoutTools.centerOnScreen(frame);
        frame.setVisible(true);
    }

    public static void main(String[] args) {
        new Launcher();
    }

    public void windowOpened(WindowEvent e) {
        csb.activate();
    }
}
