/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.placide.beans;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.Vector;
import java.util.Date;
import java.text.SimpleDateFormat;

import javax.swing.Action;
import javax.swing.JPanel;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JFrame;

import net.jourand.ally.util.ExtensionFileFilter;
import net.jourand.olaf.tools.LayoutTools;
import net.jourand.placide.places.Place;
import net.jourand.placide.places.Region;
import net.jourand.placide.places.Departement;
import net.jourand.placide.places.Arrondissement;
import net.jourand.placide.places.Canton;
import net.jourand.placide.places.Commune;
import net.jourand.placide.places.HistoricalCommune;
import net.jourand.placide.tools.Finder;
import net.jourand.placide.widgets.CommuneSearchPanelListener;
import net.jourand.placide.widgets.CommuneSearchPanel;
import net.jourand.placide.widgets.HistoryDialog;

public class CommuneSearchBean extends WindowAdapter implements CommuneSearchPanelListener, ActionListener {
  private static String[] links    = {"de ", "d'", "du ", "de la ", "des ", "de l'", "des ", "de las ", "de los "};
  private static String[] articles = {"", "", "Le ", "La ", "Les ", "L'", "Aux ", "Las ", "Los "};

  private Region[]            regions            = null;
  private Departement[]       departements       = null;
  private Arrondissement[]    arrondissements    = null;
  private Canton[]            cantons            = null;
  private Commune[]           communes           = null;
  private HistoricalCommune[] histories          = null;
  private Commune[]           searchables        = null;
  private CommuneSearchPanel  panel              = null;
  private Commune[]           result             = null;
  private Finder              finder             = null;
  private JFrame              owner              = null;
  private Commune             selected           = null;
  private SimpleDateFormat    sdf                = null;
  private Vector              searches           = null;
  private CommuneSearch       current            = null;
  private NewSearchAction     newSearchAction    = null;
  private SaveAction          saveAction         = null;
  private ResultCopyAction    copyAction         = null;
  private ClearSearchAction   clearSearchAction  = null;
  private DeleteSearchAction  deleteSearchAction = null;
  private NavigationAction    previousAction     = null;
  private NavigationAction    nextAction         = null;
  private NavigationAction    currentAction      = null;

  public CommuneSearchBean()  {
    panel = new CommuneSearchPanel();
    newSearchAction = new NewSearchAction(this);
    panel.setNewSearchAction(newSearchAction);
    saveAction = new SaveAction(this);
    panel.setSaveAction(saveAction);
    copyAction = new ResultCopyAction();
    panel.setCopyAction(copyAction);
    deleteSearchAction = new DeleteSearchAction(this);
    panel.setDeleteSearchAction(deleteSearchAction);
    clearSearchAction = new ClearSearchAction(this);
    panel.setClearSearchAction(clearSearchAction);
    previousAction = new NavigationAction(this, null);
    previousAction.setEnabled(false);
    panel.setPreviousAction(previousAction);
    nextAction = new NavigationAction(this, null);
    nextAction.setEnabled(false);
    panel.setNextAction(nextAction);
    panel.addCommuneSearchPanelListener(this);
    finder = new Finder();
    sdf = new SimpleDateFormat("dd/MM/yyyy");
    searches = new Vector();
  }

  public void setOwner(JFrame owner) {
    this.owner = owner;
    panel.setOwner(owner);
  }

  public void setRegions(Region[] regions) {
    this.regions = regions;
    finder.setRegions(regions);
  }

  public void setDepartements(Departement[] departements) {
    this.departements = departements;
    finder.setDepartements(departements);
  }

  public void setArrondissements(Arrondissement[] arrondissements) {
    this.arrondissements = arrondissements;
    finder.setArrondissements(arrondissements);
  }

  public void setCantons(Canton[] cantons) {
    this.cantons = cantons;
    finder.setCantons(cantons);
  }

  public void setCommunes(Commune[] communes) {
    this.communes = communes;
    finder.setCommunes(communes);
  }

  public void setHistories(HistoricalCommune[] histories) {
    this.histories = histories;
    finder.setHistories(histories);
  }

  public void setSearchables(Commune[] searchables) {
    this.searchables = searchables;
    finder.setSearchables(searchables);
  }

  public JPanel getPanel() {
    return panel;
  }

  public void activate() {
    panel.activate();
  }

  public void searchRequested(String namePart) {
    int index = 0;

    if(currentAction != null) {
        panel.insertPreviousAction(currentAction, 0);
        if(searches.size() > 0) {
          panel.setPreviousEnabled(true);
          index = searches.indexOf(current);
          for(int i = index+1; i < searches.size(); i++) {
            currentAction = (NavigationAction)panel.removeNextAction(0);
            panel.insertPreviousAction(currentAction, 0);
          }
        }
    }
    clearSearchAction.setEnabled(true);
    deleteSearchAction.setEnabled(true);
    panel.setNextEnabled(false);
    current = new CommuneSearch();
    current.setNamePart(namePart);
    currentAction = new NavigationAction(this, current);
    if((current.getNamePart() != null) && (!"".equals(current.getNamePart()))) {
      currentAction.putValue(currentAction.NAME, current.getNamePart());
    } else {
      currentAction.putValue(currentAction.NAME, "-  sans critre  -");
    }
    search(namePart);
    searches.addElement(current);
  }

  private void search(String namePart) {
    Place      parent = null;
    String[][] values = null;

    owner.setCursor(new Cursor(Cursor.WAIT_CURSOR));
    result = finder.searchCommunesWithNameContaining(namePart);
    if (result != null) {
      values = new String[result.length][3];
      for(int i=0; i<result.length; i++) {
        values[i][0] = result[i].getArticle();
        values[i][1] = result[i].getName();
        parent = result[i].getParent();
        if(parent.getType() == Place.DEPARTEMENT) {
          values[i][2] = parent.getCode();
        } else {
          values[i][2] = parent.getParent().getParent().getCode();
        }
      }
      if(result.length == 0) {
        copyAction.setValue(null);
        saveAction.setEnabled(false);
      } else {
        saveAction.setEnabled(true);
      }
      panel.setResults(values, current.getSelected());
    }
    owner.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
  }

  public void resultSelectionChanged(int index) {
    String[]            details   = null;
    Place               parent    = null;
    short               idx       = 2;
    short               iconIndex = -1;
    boolean             modified  = false;
    HistoricalCommune[] history   = null;
    String              copyValue = null;

    details = new String[] {" "," "," "," "," ", " ", " ", " ", " "};
    if((index < result.length) && (index >= 0)) {
      current.setSelected(index);
      copyAction.setEnabled(true);
      selected = result[index];
      copyValue = articles[selected.getNameType()]+selected.getName()+", ";
      iconIndex = (short)(selected.getActual());
      switch(selected.getActual()) {
        case 0:
          details[0] = "Ancien nom";
          details[1] = "La commune "
                     + links[selected.getNameType()]
                     + " "
                     + selected.getName()
                     + " est situe dans :";
          break;
        case 1:
          details[0] = "Commune vivante";
          details[1] = "La commune "
                     + links[selected.getNameType()]
                     + " "
                     + selected.getName()
                     + " est situe dans :";
          break;
        case 2:
          details[0] = "Commune \"associe\"";
          details[1] = "La commune "
                     + links[selected.getNameType()]
                     + " "
                     + selected.getName()
                     + " est situe dans :";
          break;
        case 3:
          details[0] = "Commune prime";
          details[1] = "La commune "
                     + links[selected.getNameType()]
                     + " "
                     + selected.getName()
                     + " tait situe dans :";
          break;
        case 4:
          details[0] = "Ancien code d  un changement de dpartement";
          details[1] = "La commune "
                     + links[selected.getNameType()]
                     + " "
                     + selected.getName()
                     + " tait situe dans :";
          break;
        case 5:
          details[0] = "Arrondissement municipal";
          details[1] = "L'arrondissement municipal "
                     + links[selected.getNameType()]
                     + " "
                     + selected.getName()
                     + " est situe dans :";
          break;
        case 6:
          details[0] = "Fraction cantonale";
          details[1] = "La fraction cantonale "
                     + links[selected.getNameType()]
                     + " "
                     + selected.getName()
                     + " est situe dans :";
          break;
      }
      parent = (Place)selected.getParent();
      do {
        switch(parent.getType()) {
          case Place.CANTON:
            details[idx] = "   - le canton "
                         + links[parent.getNameType()]
                         + " "
                         + parent.getName();
            break;
          case Place.ARRONDISSEMENT:
            details[idx] = "   - l'arrondissement "
                         + links[parent.getNameType()]
                         + " "
                         + parent.getName();
            break;
          case Place.DEPARTEMENT:
            details[idx] = "   - le dpartement "
                         + links[parent.getNameType()]
                         + " "
                         + parent.getName();
            copyValue += parent.getName()+", ";
            break;
          case Place.REGION:
            details[idx] = "   - la rgion "
                         + links[parent.getNameType()]
                         + " "
                         + parent.getName();
            copyValue += parent.getName()+", ";
            break;
        }
        idx++;
        parent = (Place)parent.getParent();
      } while(parent != null);
      copyValue += "FRANCE";
      switch(selected.getActual()) {
        case 2:
          details[6] = "Elle est \"associe\"  la commune "
                     + links[selected.getPole().getNameType()]
                     + " "
                     + selected.getPole().getName();
          break;
        case 3:
          details[6] = "Elle a fusionn avec la commune "
                     + links[selected.getPole().getNameType()]
                     + " "
                     + selected.getPole().getName();
          break;
        case 4:
          details[6] = "Son nouveau code INSEE est "
                     + selected.getPole().getCode();
          break;
        case 5:
          details[6] = "Il appartient  la commune "
                     + links[selected.getPole().getNameType()]
                     + " "
                     + selected.getPole().getName();
          break;
      }
      details[7] = "Code INSEE : "+selected.getCode();
      if (selected.getPostalCode() != null) {
        details[8] = "Code postal : "+selected.getPostalCode();
      }
      history = finder.findHistoricalCommuneWithCode(selected.getCode());
      if((history != null) && (history.length > 0)) {
        modified = true;
      }
    } else {
      copyAction.setEnabled(false);
      selected = null;
      copyValue = null;
    }
    panel.setDetails(details, iconIndex, modified);
    copyAction.setValue(copyValue);
  }

  public void historyRequested() {
    HistoryDialog       hd             = null;
    HistoricalCommune[] history        = null;
    Date                date           = null;
    String              title          = null;
    String              legal          = null;
    String              jo             = null;
    String              effective      = null;
    Vector              description    = null;
    Commune             commune        = null;
    String              str            = null;
    int                 n              = 0;
    Departement         departement    = null;
    Arrondissement      arrondissement = null;
    Canton              canton         = null;

    history = finder.findHistoricalCommuneWithCode(selected.getCode());
    hd = new HistoryDialog(owner, "Historique "+links[selected.getNameType()]+" "+selected.getName());
    hd.addWindowListener(this);
    for(int i=0; i<history.length; i++) {
      date = history[i].getMostRecentDate();
      title = null;
      legal = null;
      jo = null;
      effective = null;
      description = new Vector();
      switch(history[i].getLegalType()) {
        case 0:
          legal = "Dcret";
          break;
        case 1:
          legal = "Ordonnance";
          break;
        case 2:
          legal = "Loi";
          break;
        case 3:
          legal = "Arrt";
          break;
        case 4:
          legal = "Dcision du Conseil d'Etat";
          break;
        case 5:
          legal = "Dcision de la commission de rvision du nom des communes";
          break;
        case 6:
          legal = "Erreur de modification due  une erreur de l'INSEE";
          break;
      }
      if(history[i].getLegalDate() != null) {
        legal += " en date du "+sdf.format(history[i].getLegalDate());
      }
      if(history[i].getJoDate() != null) {
        jo = "Parution au journal officiel du "+sdf.format(history[i].getJoDate());
      }
      if(history[i].getEffectiveDate() != null) {
        effective = "Date effective : "+sdf.format(history[i].getEffectiveDate());
      }
      switch(history[i].getModification()) {
        case 10:
          title = "Changement de nom";
          description.addElement(articles[history[i].getNameType()]+history[i].getName()
                              + " devient "
                              + articles[history[i].getNewNameType()]+history[i].getNewName());
          break;
        case 11:
          description.addElement(articles[history[i].getNameType()]+history[i].getName()
                              + " devient "
                              + articles[history[i].getNewNameType()]+history[i].getNewName());
          switch(history[i+1].getModification()) {
            case 31:
              title = "Changement de nom d  une fusion";
              commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
              description.addElement("Fusion avec "+articles[commune.getNameType()]+commune.getName());
              break;
            case 32:
              title = "Changement de nom d  une fusion";
              description.addElement("Commune absorbante de la fusion avec :");
              for(i++; (i<history.length)
                    && (date.equals(history[i].getMostRecentDate()))
                    && (history[i].getModification()==32); i++) {
                commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
                description.addElement("   - "+articles[commune.getNameType()]+commune.getName());
              }
              i--;
              break;
            case 33:
              title = "Changement de nom d  une fusion-association";
              commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
              description.addElement("Fusion-association avec "+articles[commune.getNameType()]+commune.getName());
              break;
            case 34:
              title = "Changement de nom d  une fusion-association";
              description.addElement("Commune absorbante de la fusion-association avec :");
              for(i++; (i<history.length)
                    && (date.equals(history[i].getMostRecentDate()))
                    && (history[i].getModification()==34); i++) {
                commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
                description.addElement("   - "+articles[commune.getNameType()]+commune.getName());
              }
              i--;
              break;
            case 35:
              title = "Changement de nom d  une fusion-association se transformant en fusion simple";
              commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
              description.addElement("Fusion-association se transformant en fusion simple avec "+articles[commune.getNameType()]+commune.getName());
              break;
            case 36:
              title = "Changement de nom d  une fusion-association se transformant en fusion simple";
              description.addElement("Commune absorbante de la fusion-association se transformant en fusion simple avec :");
              for(i++; (i<history.length)
                    && (date.equals(history[i].getMostRecentDate()))
                    && (history[i].getModification()==36); i++) {
                commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
                description.addElement("   - "+articles[commune.getNameType()]+commune.getName());
              }
              i--;
              break;
          }
          break;
        case 12:
          title = "Changement de nom d  un rtablissement";
          description.addElement(articles[history[i].getNameType()]+history[i].getName()
                              + " devient "
                              + articles[history[i].getNewNameType()]+history[i].getNewName());
          i++;
          if((i<history.length)
             && (date.equals(history[i].getMostRecentDate()))
             && (history[i].getModification()==21)) {
            commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
            description.addElement("Commune spare d'avec "+articles[commune.getNameType()]+commune.getName());
          } else if((i<history.length)
             && (date.equals(history[i].getMostRecentDate()))
             && (history[i].getModification()==23)) {
            commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
            description.addElement("Commune se sparant d'avec "+articles[commune.getNameType()]+commune.getName());
          } else {
            i--;
          }
          break;
        case 14:
          title = "Changement de nom d au transfert du chef-lieu";
          description.addElement(articles[history[i].getNameType()]+history[i].getName()
                              + " devient "
                              + articles[history[i].getNewNameType()]+history[i].getNewName());
          if(((i+1)<history.length)
             && (history[i+1].getModification() == 22)
             && (date.equals(history[i+1].getMostRecentDate()))) {
            i++;
            commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
            description.addElement("Commune ayant donn des parcelles pour la cration "
                                + links[commune.getNameType()]+commune.getName());
          }
          break;
        case 20:
          title = "Cration";
          description.addElement("Communes ayant donn des parcelles pour la cration :");
          for(; (i<history.length)
                && (date.equals(history[i].getMostRecentDate()))
                && (history[i].getModification()==20); i++) {
            commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
            description.addElement("   - "+articles[commune.getNameType()]+commune.getName());
          }
          i--;
          break;
        case 21:
          title = "Rtablissement";
          commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
          description.addElement("Commune spare d'avec "+articles[commune.getNameType()]+commune.getName());
          break;
        case 22:
          title = "Commune ayant donn des parcelles pour une cration";
          commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
          description.addElement("Commune ayant donn des parcelles pour la cration "
                              + links[commune.getNameType()]+commune.getName());
          break;
        case 23:
          title = "Commune se sparant";
          commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
          description.addElement("Sparation d'avec "+articles[commune.getNameType()]+commune.getName());
          break;
        case 24:
          title = "Cration de fractions cantonales";
          for(n=0; (i<history.length) && (date.equals(history[i].getMostRecentDate())) && (history[i].getModification()==24); i++) {
            n++;
          }
          i--;
          description.addElement("Nombre de fractions cantonales cres : "+n);
          break;
        case 30:
          title = "Suppression";
          description.addElement("Communes ayant reu des parcelles :");
          for(; (i<history.length) && (date.equals(history[i].getMostRecentDate())) && (history[i].getModification()==30); i++) {
            commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
            description.addElement("   - "+articles[commune.getNameType()]+commune.getName());
          }
          i--;
          break;
        case 31:
          title = "Fusion";
          commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
          description.addElement("Fusion avec "+articles[commune.getNameType()]+commune.getName());
          break;
        case 32:
          title = "Fusion";
          description.addElement("Commune absorbante de la fusion avec :");
          for(; (i<history.length) && (date.equals(history[i].getMostRecentDate())) && (history[i].getModification()==32); i++) {
            commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
            description.addElement("   - "+articles[commune.getNameType()]+commune.getName());
          }
          i--;
          break;
        case 33:
          title = "Fusion-association";
          commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
          description.addElement("Fusion-association avec "+articles[commune.getNameType()]+commune.getName());
          break;
        case 34:
          title = "Fusion-association";
          description.addElement("Commune absorbante de la fusion-association avec :");
          for(; (i<history.length) && (date.equals(history[i].getMostRecentDate())) && (history[i].getModification()==34); i++) {
            commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
            description.addElement("   - "+articles[commune.getNameType()]+commune.getName());
          }
          i--;
          break;
        case 35:
          title = "Fusion-association se transformant en fusion simple";
          commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
          description.addElement("Fusion-association se transformant en fusion simple avec "+articles[commune.getNameType()]+commune.getName());
          break;
        case 36:
          title = "Fusion-association se transformant en fusion simple";
          description.addElement("Commune-ple de la fusion-association qui s'est transforme en fusion simple avec :");
          for(; (i<history.length) &&  (date.equals(history[i].getMostRecentDate())) && (history[i].getModification()==36); i++) {
            commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
            description.addElement("   - "+articles[commune.getNameType()]+commune.getName());
          }
          i--;
          break;
        case 37:
          title = "Suppression de fractions cantonales";
          for(n=0; (i<history.length) && (date.equals(history[i].getMostRecentDate())) && (history[i].getModification()==37); i++) {
            n++;
          }
          i--;
          description.addElement("Nombre de fractions cantonales supprimes : "+n);
          break;
        case 39:
          title = "Rception de parcelle suite  une suppression";
          commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
          description.addElement("Commune ayant reu des parcelles "+links[commune.getNameType()]+commune.getName());
          break;
        case 40:
          // No 40 found in the history file !
          break;
        case 41:
          title = "Changement de dpartement";
          departement = finder.findDepartementWithCode(history[i].getOldCode().substring(0,2));
          str = "Passage du dpartement "
              + links[departement.getNameType()]+departement.getName();
          departement = finder.findDepartementWithCode(history[i].getCode().substring(0,2));
          str += " vers celui "
              + links[departement.getNameType()]+departement.getName();
          description.addElement(str);
          break;
        case 42:
          title = "Changement d'arrondissement";
          arrondissement = finder.findArrondissementWithLongCode(history[i].getOldArrondissementCode());
          str = "Passage de l'arrondissement "
              + links[arrondissement.getNameType()]+arrondissement.getName();
          arrondissement = (Arrondissement)selected.getParent().getParent();
          str += "  l'arrondissement "
              + links[arrondissement.getNameType()]+arrondissement.getName();
          description.addElement(str);
          break;
        case 43:
          if(((i+1)<history.length)
             && (history[i+1].getModification() == 42)
             && (date.equals(history[i+1].getMostRecentDate()))) {
            title = "Changement d'arrondissement et de canton";
            arrondissement = finder.findArrondissementWithLongCode(history[i+1].getOldArrondissementCode());
            str = "Passage de l'arrondissement "
                + links[arrondissement.getNameType()]+arrondissement.getName();
            arrondissement = (Arrondissement)selected.getParent().getParent();
            str += "  l'arrondissement "
                + links[arrondissement.getNameType()]+arrondissement.getName();
            description.addElement(str);
            canton = finder.findCantonWithLongCode(history[i].getOldCantonCode());
            description.addElement("Passage du canton "+links[canton.getNameType()]+canton.getName()
                                + " au canton "
                                + links[selected.getParent().getNameType()]+selected.getParent().getName());
            i++;
          } else {
            title = "Changement de canton";
            canton = finder.findCantonWithLongCode(history[i].getOldCantonCode());
            description.addElement("Passage du canton "+links[canton.getNameType()]+canton.getName()
                                + " au canton "
                                + links[selected.getParent().getNameType()]+selected.getParent().getName());
          }
          break;
        case 50:
          if(((i+1)<history.length)
             && (history[i+1].getModification() == 13)
             && (date.equals(history[i+1].getMostRecentDate()))) {
            title = "Changement de nom d au changement de nom du chef-lieu";
            i++;
            description.addElement(articles[history[i].getNameType()]+history[i].getName()
                                + " devient "
                                + articles[history[i].getNewNameType()]+history[i].getNewName());
            commune = finder.findCommuneWithCode(history[i-1].getOldChefLieuCode());
            str = "Transfert de chef-lieu de commune "
                + links[commune.getNameType()]+commune.getName();
            commune = finder.findCommuneWithCode(history[i-1].getChefLieuCode());
            str += " vers "
                + articles[commune.getNameType()]+commune.getName();
          } else {
            title = "Transfert de chef-lieu de commune";
            commune = finder.findCommuneWithCode(history[i].getOldChefLieuCode());
            str = "Transfert "
                + links[commune.getNameType()]+commune.getName();
            commune = finder.findCommuneWithCode(history[i].getChefLieuCode());
            str += " vers "
                + articles[commune.getNameType()]+commune.getName();
          }
          description.addElement(str);
          break;
        case 51:
          if(((i+1)<history.length)
             && (history[i+1].getModification() == 43)
             && (date.equals(history[i+1].getMostRecentDate()))) {
            title = "Transfert de chef-lieu de canton et changement de canton";
            commune = finder.findCommuneWithCode(history[i].getOldChefLieuCode());
            str = "Transfert de chef-lieu "
                + links[commune.getNameType()]+commune.getName()
                + " vers ";
            commune = finder.findCommuneWithCode(history[i].getChefLieuCode());
            str += articles[commune.getNameType()]+commune.getName();
            description.addElement(str);
            i++;
          } else {
            title = "Transfert de chef-lieu de canton";
            commune = finder.findCommuneWithCode(history[i].getOldChefLieuCode());
            str = "Transfert "
                + links[commune.getNameType()]+commune.getName()
                + " vers ";
            commune = finder.findCommuneWithCode(history[i].getChefLieuCode());
            str += articles[commune.getNameType()]+commune.getName();
            description.addElement(str);
          }
          break;
        case 52:
          title = "Transfert de chef-lieu d'arrondissement";
          commune = finder.findCommuneWithCode(history[i].getOldChefLieuCode());
          str = "Transfert "
              + links[commune.getNameType()]+commune.getName()
              + " vers ";
          commune = finder.findCommuneWithCode(history[i].getChefLieuCode());
          str += articles[commune.getNameType()]+commune.getName();
          description.addElement(str);
          break;
        case 53:
          // No 53 found in the history file !
          break;
        case 54:
          // No 54 found in the history file !
          break;
        case 60:
        case 61:
        case 62:
        case 63:
          title = "Trensfert de parcelles";
          for(;(i<history.length)
             && ((history[i].getModification() == 60)
              || (history[i].getModification() == 61)
              || (history[i].getModification() == 62)
              || (history[i].getModification() == 63))
             && (date.equals(history[i].getMostRecentDate())); i++) {
            switch(history[i].getModification()) {
              case 60:
                str = "Cession de parcelles avec incidence dmographique  la commune ";
                break;
              case 61:
                str = "Cession de parcelles sans incidence dmographique  la commune ";
                break;
              case 62:
                str = "Rception de parcelles avec incidence dmographique de la commune ";
                break;
              case 63:
                str = "Rception de parcelles sans incidence dmographique de la commune ";
                break;
            }
            commune = finder.findCommuneWithCode(history[i].getFusionnedCode());
            str += links[commune.getNameType()]+commune.getName();
            description.addElement(str);
          }
          i--;
          break;
        case 70:
          // No 70 found in the history file !
          break;
        default:
          title = "Non implment ("+history[i].getModification()+")";
      }
      hd.add(date, title, legal, jo, effective, description);
    }
    hd.pack();
    LayoutTools.centerOnWindow(hd, owner);
    hd.setVisible(true);
  }

  public void actionPerformed(ActionEvent e) {
    Object       source = null;
    int          index  = 0;
    int          target = 0;
    JFileChooser jfc    = null;
    String       name   = null;
    boolean      ok     = false;

    source = e.getSource();
    if(source == clearSearchAction) {
      for(int i = 0, b = 0; i < searches.size(); i++) {
        if (searches.elementAt(i) == current) {
          b = 1;
        } else {
          if(b == 0) {
            panel.removePreviousAction(0);
          } else {
            panel.removeNextAction(0);
          }
        }
      }
      searches = new Vector();
      current = null;
      currentAction = null;
      saveAction.setEnabled(false);
      deleteSearchAction.setEnabled(false);
      clearSearchAction.setEnabled(false);
      panel.setNextEnabled(false);
      panel.setPreviousEnabled(false);
      panel.setNamePart("");
      panel.setResults(new String[0][0], -1);
      panel.activate();
    } else if(source == newSearchAction) {
      deleteSearchAction.setEnabled(false);
      for(int i = 0, b = 0; i < searches.size(); i++) {
        if (searches.elementAt(i) == current) {
          panel.setPreviousEnabled(true);
          panel.insertPreviousAction(currentAction, 0);
          b = 1;
        } else if(b == 1) {
          currentAction = (NavigationAction)panel.removeNextAction(0);
          panel.insertPreviousAction(currentAction, 0);
        }
      }
      current = null;
      currentAction = null;
      saveAction.setEnabled(false);
      panel.setNextEnabled(false);
      panel.setNamePart("");
      panel.setResults(new String[0][0], -1);
      panel.activate();
    } else if(source == deleteSearchAction) {
      index = searches.indexOf(current);
      if(index > 0) {
        currentAction = (NavigationAction)panel.removePreviousAction(0);
        current = (CommuneSearch)searches.elementAt(index-1);
      } else if(searches.size() > 1) {
        currentAction = (NavigationAction)panel.removeNextAction(0);
        current = (CommuneSearch)searches.elementAt(index+1);
      } else {
        currentAction = null;
        current = null;
      }
      searches.removeElementAt(index);
      if(current != null) {
        target = searches.indexOf(current);
        search(current.getNamePart());
        panel.setNamePart(current.getNamePart());
        if(target == 0) {
          panel.setPreviousEnabled(false);
        }
        if(target == searches.size()-1) {
          panel.setNextEnabled(false);
        }
      } else {
        saveAction.setEnabled(false);
        copyAction.setEnabled(false);
        deleteSearchAction.setEnabled(false);
        clearSearchAction.setEnabled(false);
        panel.setPreviousEnabled(false);
        panel.setNextEnabled(false);
        panel.setNamePart("");
        panel.setResults(new String[0][0], -1);
        panel.activate();
      }
    } else if(source == saveAction) {
      jfc = new JFileChooser();
      jfc.setFileFilter(new ExtensionFileFilter("txt", "Fichiers texte (*.txt)"));
      do {
        if(jfc.showSaveDialog(owner) == JFileChooser.APPROVE_OPTION) {
          name = jfc.getSelectedFile().getAbsolutePath();
          if(!name.endsWith(".txt")) {
            name += ".txt";
          }
          if((new File(name)).exists()) {
            if(JOptionPane.showConfirmDialog(owner,
               "Un fichier portant le mme nom existe dj !\n Le remplacer ?",
               "Attention !", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE) == JOptionPane.YES_OPTION ) {
              ok = true;
              save(name);
            }
          } else {
            ok = true;
            save(name);
          }
        } else {
          ok = true;
        }
      } while(!ok);
    } else if(source == previousAction) {
      if(current != null) {
        index = searches.indexOf(current);
        panel.setNextEnabled(true);
        panel.insertNextAction(currentAction, 0);
      } else {
        index = searches.size();
      }
      if(index == 1) {
        panel.setPreviousEnabled(false);
      }
      currentAction = (NavigationAction)panel.removePreviousAction(0);
      current = (CommuneSearch)searches.elementAt(index-1);
      search(current.getNamePart());
      panel.setNamePart(current.getNamePart());
    } else if(source == nextAction) {
      index = searches.indexOf(current);
      if(index == searches.size()-2) {
        panel.setNextEnabled(false);
      }
      panel.setPreviousEnabled(true);
      panel.insertPreviousAction(currentAction, 0);
      currentAction = (NavigationAction)panel.removeNextAction(0);
      current = (CommuneSearch)searches.elementAt(index+1);
      search(current.getNamePart());
      panel.setNamePart(current.getNamePart());
    } else {
      if(current != null) {
        index = searches.indexOf(current);
      } else {
        index = searches.size();
      }
      target = searches.indexOf(source);
      if(target <= index) {
        if(currentAction != null) {
          panel.insertNextAction(currentAction, 0);
          panel.setNextEnabled(true);
        }
        for(int i = target+1; i < index; i++) {
          currentAction = (NavigationAction)panel.removePreviousAction(0);
          panel.insertNextAction(currentAction, 0);
          panel.setNextEnabled(true);
        }
        currentAction = (NavigationAction)panel.removePreviousAction(0);
        current = (CommuneSearch)searches.elementAt(target);
        if(target == 0) {
          panel.setPreviousEnabled(false);
        }
        search(current.getNamePart());
        panel.setNamePart(current.getNamePart());
      } else {
        panel.insertPreviousAction(currentAction, 0);
        panel.setPreviousEnabled(true);
        for(int i = index+1; i < target; i++) {
          currentAction = (NavigationAction)panel.removeNextAction(0);
          panel.insertPreviousAction(currentAction, 0);
        }
        currentAction = (NavigationAction)panel.removeNextAction(0);
        current = (CommuneSearch)searches.elementAt(target);
        if(target == searches.size()-1) {
          panel.setNextEnabled(false);
        }
        search(current.getNamePart());
        panel.setNamePart(current.getNamePart());
      }
    }
  }

  private void save(String fileName) {
    PrintWriter pw     = null;
    String      line   = null;
    Place       parent = null;

    try {
      owner.setCursor(new Cursor(Cursor.WAIT_CURSOR));
      pw = new PrintWriter(new FileOutputStream(fileName));
      for(int i=0; i<result.length; i++) {
        line = articles[result[i].getNameType()]+result[i].getName()+", ";
        parent = (Place)result[i].getParent();
        do {
          switch(parent.getType()) {
            case Place.DEPARTEMENT:
              line += parent.getName()+", ";
              break;
            case Place.REGION:
              line += parent.getName()+", ";
              break;
          }
          parent = (Place)parent.getParent();
        } while(parent != null);
        line += "FRANCE";
        pw.println(line);
      }
    } catch (FileNotFoundException fnfe) {
      JOptionPane.showMessageDialog(owner,
         "Une erreur s'est produite lors de l'enregistrement du fichier !",
         "Erreur d'enregistrement", JOptionPane.ERROR_MESSAGE);
    } finally {
      owner.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
      pw.close();
      pw = null;
    }
  }

  public void windowClosed(WindowEvent e) {
    panel.activateResult();
  }
}
