/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.placide.places;

import java.io.Serializable;


public class Place implements Serializable {
    public static final short NONE = 0;
    public static final short REGION = 1;
    public static final short DEPARTEMENT = 2;
    public static final short ARRONDISSEMENT = 3;
    public static final short CANTON = 4;
    public static final short COMMUNE = 5;
    public static final String[] articles = {
                                                "", "", "Le ", "La ", "Les ",
                                                "L'", "Aux ", "Las ", "Los "
                                            };
    public static final String[] links = {
                                             "de ", "d'", "du ", "de la ",
                                             "des ", "de l'", "des ", "de las ",
                                             "de los "
                                         };
    protected short type = NONE;
    protected String code = null;
    protected short nameType = 0;
    protected String name = null;
    protected Place parent = null;
    protected transient String canonicalName = null;
    protected Place mainPlace = null;

    public Place(short type) {
        this.type = type;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public short getNameType() {
        return nameType;
    }

    public String getArticle() {
        return articles[nameType];
    }

    public void setNameType(short nameType) {
        this.nameType = nameType;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getLongName() {
        return articles[nameType] + getName();
    }

    public Place getParent() {
        return parent;
    }

    public void setParent(Place parent) {
        this.parent = parent;
    }

    public String getCanonicalName() {
        if (canonicalName == null) {
            canonicalName = canonize(name);
        }

        return canonicalName;
    }

    public short getType() {
        return type;
    }

    public void setMainPlace(Place mainPlace) {
        this.mainPlace = mainPlace;
    }

    public Place getMainPlace() {
        return mainPlace;
    }

    public static String canonize(String string) {
        String canon = null;
        String str = null;
        char c;

        canon = "";
        str = string.toLowerCase();

        for (int j = 0; j < str.length(); j++) {
            c = str.charAt(j);

            
            if ((c == '') || (c == '') || (c == '') || (c == '')
                || (c == '') || (c == '')) {
                c = 'a';
            } else if (c == '') {
                c = 'c';
            } else if ((c == '') || (c == '') || (c == '') || (
                           c == ''
                       )) {
                c = 'e';
            } else if ((c == '') || (c == '') || (c == '') || (
                           c == ''
                       )) {
                c = 'i';
            } else if (c == '') {
                c = 'n';
            } else if ((c == '') || (c == '') || (c == '') || (
                           c == ''
                       ) || (c == '')) {
                c = 'o';
            } else if ((c == '') || (c == '') || (c == '') || (
                           c == ''
                       )) {
                c = 'u';
            } else if ((c == '') || (c == '')) {
                c = 'y';
            } else if (c == '') {
                canon += 'a';
                c = 'e';
            } else if (c == '') {
                canon += 'o';
                c = 'e';
            } else if (c == '') {
                c = 's';
            } else if (c == '') {
                canon += 's';
                c = 's';
            } else if ((c == '\'') || (c == '-')) {
            	c= ' ';
            }

            canon += c;
        }

        return canon;
    }
}
