/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.placide.tools;

import java.util.Vector;

import net.jourand.placide.places.Place;
import net.jourand.placide.places.Region;
import net.jourand.placide.places.Departement;
import net.jourand.placide.places.Arrondissement;
import net.jourand.placide.places.Canton;
import net.jourand.placide.places.Commune;
import net.jourand.placide.places.HistoricalCommune;

public class Finder {
  private Region[]            regions         = null;
  private Departement[]       departements    = null;
  private Arrondissement[]    arrondissements = null;
  private Canton []           cantons         = null;
  private Commune[]           communes        = null;
  private HistoricalCommune[] histories       = null;
  private Commune[]           searchables     = null;

  public void setRegions(Region[] regions) {
    this.regions = regions;
  }

  public void setDepartements(Departement[] departements) {
    this.departements = departements;
  }

  public void setArrondissements(Arrondissement[] arrondissements) {
    this.arrondissements = arrondissements;
  }

  public void setCantons(Canton[] cantons) {
    this.cantons = cantons;
  }

  public void setCommunes(Commune[] communes) {
    this.communes = communes;
  }

  public void setHistories(HistoricalCommune[] histories) {
    this.histories = histories;
  }

  public void setSearchables(Commune[] searchables) {
    this.searchables = searchables;
  }

  public Commune[] searchCommunesWithNameContaining(String namePart) {
    Vector    coms     = null;
    Commune[] result   = null;
    String    searched = null;

    searched = Place.canonize(namePart);
    coms = new Vector();
    for(int i=0; i<searchables.length; i++) {
      if (searchables[i].getSearchableName().indexOf(searched) != -1) {
        coms.addElement(searchables[i]);
      }
    }

    if (coms.size() > 0) {
      result = (Commune[])coms.toArray((Object[])new Commune[0]);
    } else {
      result = new Commune[0];
    }

    return result;
  }

  public Departement findDepartementWithCode(String code) {
    Departement departement = null;

    for(int i=0; (i<departements.length) && (departement==null); i++) {
      if (departements[i].getCode().equals(code)) {
        departement = departements[i];
      }
    }

    return departement;
  }

  public Departement findDepartementWithName(String name) {
    Departement departement = null;

    for(int i=0; (i<departements.length) && (departement==null); i++) {
      if (departements[i].getName().equals(name)) {
        departement = departements[i];
      }
    }

    return departement;
  }

  public Region findRegionWithCode(String code) {
    Region region = null;

    for(int i=0; (i<regions.length) && (region==null); i++) {
      if (regions[i].getCode().equals(code)) {
        region = regions[i];
      }
    }

    return region;
  }

  public HistoricalCommune[] findHistoricalCommuneWithCode(String code) {
    Vector              hCommunes = null;
    HistoricalCommune[] result    = null;

    hCommunes = new Vector();
    for(int i=0; i<histories.length; i++) {
      if (code.equals(histories[i].getCode())) {
        hCommunes.addElement(histories[i]);
      }
    }

    if (hCommunes.size() > 0) {
      result = (HistoricalCommune[])hCommunes.toArray((Object[])new HistoricalCommune[0]);
    } else {
      result = new HistoricalCommune[0];
    }

    return result;
  }

  public Commune findCommuneWithCode(String code) {
    Commune commune = null;

    for(int i=0; (i<communes.length) && (commune==null); i++) {
      if (code.equals(communes[i].getCode())) {
        commune = communes[i];
      }
    }

    return commune;
  }

  public Arrondissement findArrondissementWithLongCode(String code) {
    Arrondissement arrondissement = null;

    for(int i=0; (i<arrondissements.length) && (arrondissement==null); i++) {
      if(code.equals(arrondissements[i].getParent().getCode()+arrondissements[i].getCode())) {
        arrondissement = arrondissements[i];
      }
    }

    return arrondissement;
  }

  public Canton findCantonWithLongCode(String code) {
    Canton canton = null;

    for(int i=0; (i<cantons.length) && (canton==null); i++) {
      if(code.equals(cantons[i].getParent().getParent().getCode()+cantons[i].getCode())) {
        canton = cantons[i];
      }
    }

    return canton;
  }

/*
  public Commune[] findCommunesWithNameContaining(String namePart) {
    Vector    coms     = null;
    Commune[] result   = null;
    String    searched = null;

    searched = Canonizer.canonize(namePart);
    coms = new Vector();
    for(int i=0; i<communes.length; i++) {
      if (communes[i].getCanonicalName().indexOf(searched) != -1) {
        coms.addElement(communes[i]);
      }
    }

    if (coms.size() > 0) {
      result = (Commune[])coms.toArray((Object[])new Commune[0]);
    } else {
      result = new Commune[0];
    }

    return result;
  }

  public Commune[] findCommunesWithLongNameStartingWith(String namePart) {
    Vector    coms     = null;
    Commune[] result   = null;
    String    searched = null;
    int       idx      = 0;

    searched = Canonizer.canonize(namePart);
    coms = new Vector();
    for(int i=0; i<communes.length; i++) {
      if (communes[i].getCanonicalLongName().startsWith(searched)) {
        coms.addElement(communes[i]);
      }
    }

    if (coms.size() > 0) {
      result = (Commune[])coms.toArray((Object[])new Commune[0]);
    } else {
      result = new Commune[0];
    }

    return result;
  }

  public Commune[] findCommunesWithNameStartingWith(String namePart) {
    Vector    coms     = null;
    Commune[] result   = null;
    String    searched = null;
    int       idx      = 0;

    searched = Canonizer.canonize(namePart);
    coms = new Vector();
    for(int i=0; i<communes.length; i++) {
      if (communes[i].getCanonicalName().startsWith(searched)) {
        coms.addElement(communes[i]);
      }
    }

    if (coms.size() > 0) {
      result = (Commune[])coms.toArray((Object[])new Commune[0]);
    } else {
      result = new Commune[0];
    }

    return result;
  }
  */
}
