/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.placide.widgets;

import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.Vector;

import javax.swing.InputMap;
import javax.swing.ActionMap;
import javax.swing.Action;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JScrollPane;
import javax.swing.JCheckBox;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.ImageIcon;
import javax.swing.ListSelectionModel;
import javax.swing.JSeparator;
import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.border.BevelBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import com.jgoodies.plaf.windows.ExtWindowsToolBarUI;

import net.jourand.olaf.tools.LayoutTools;
import net.jourand.olaf.widgets.DropDownButton;;

public class CommuneSearchPanel extends JPanel implements ActionListener, ListSelectionListener {
  private static int WIDTH = 350;

  private JButton        newSearchButton    = null;
  private JButton        saveButton         = null;
  private JButton        copyButton         = null;
  private JButton        deleteSearchButton = null;
  private JButton        clearSearchButton  = null;
  private DropDownButton previousButton     = null;
  private DropDownButton nextButton         = null;
  private JButton        aboutButton        = null;
  private JButton        licenseButton      = null;
  private JTextField     namePartTextField  = null;
  private JButton        searchButton       = null;
  private JList          resultList         = null;
  private JLabel         resultNumberLabel  = null;
  private JLabel[]       details            = null;
  private Vector         listeners          = null;
  private JButton        historyButton      = null;
  private ImageIcon[]    detailsIcons       = null;
  private InputMap       inputMap           = null;
  private ActionMap      actionMap          = null;
  private JFrame         owner              = null;

  public CommuneSearchPanel() {
    super(new GridBagLayout());

    int          idx               = 0;
    Border       b                 = null;
    TitledBorder tb                = null;
    JToolBar     toolBar           = null;
    JPanel       toolPanel         = null;
    JPanel       searchPanel       = null;
    JPanel       resultPanel       = null;
    JScrollPane  resultPane        = null;
    JPanel       detailPanel       = null;
    JPanel       topDetailPanel    = null;
    JPanel       bottomDetailPanel = null;
    JLabel       label             = null;
    ClassLoader  classLoader       = null;
    ImageIcon    icon              = null;

    listeners = new Vector();

    classLoader = getClass().getClassLoader();

    LayoutTools.constrains(this, new JSeparator(JSeparator.HORIZONTAL), 0, 0, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.SOUTH,
                        1.0, 0.0, 0, 0, 0, 0);

    inputMap = new InputMap();
    actionMap = new ActionMap();

    toolPanel = new JPanel(new GridBagLayout());
    toolBar = new JToolBar();
    toolBar.setRollover(true);
    newSearchButton = new JButton();
    newSearchButton.setFocusPainted(false);
    newSearchButton.setPreferredSize(new Dimension(24, 24));
    newSearchButton.setMinimumSize(new Dimension(24, 24));
    toolBar.add(newSearchButton);
    toolBar.addSeparator(new Dimension(2, 0));
    saveButton = new JButton();
    saveButton.setFocusPainted(false);
    saveButton.setPreferredSize(new Dimension(24, 24));
    saveButton.setMinimumSize(new Dimension(24, 24));
    toolBar.add(saveButton);
    toolBar.addSeparator(new Dimension(2, 0));
    copyButton = new JButton();
    copyButton.setFocusPainted(false);
    copyButton.setEnabled(false);
    copyButton.setPreferredSize(new Dimension(24, 24));
    copyButton.setMinimumSize(new Dimension(24, 24));
    toolBar.add(copyButton);
    toolBar.addSeparator(new Dimension(10, 0));
    previousButton = new DropDownButton();
    previousButton.setEnabled(false);
    toolBar.add(previousButton);
    toolBar.addSeparator(new Dimension(2, 0));
    nextButton = new DropDownButton();
    nextButton.setEnabled(false);
    toolBar.add(nextButton);
    toolBar.addSeparator(new Dimension(2, 0));
    deleteSearchButton = new JButton();
    deleteSearchButton.setFocusPainted(false);
    deleteSearchButton.setPreferredSize(new Dimension(24, 24));
    deleteSearchButton.setMinimumSize(new Dimension(24, 24));
    toolBar.add(deleteSearchButton);
    toolBar.addSeparator(new Dimension(2, 0));
    clearSearchButton = new JButton();
    clearSearchButton.setFocusPainted(false);
    clearSearchButton.setEnabled(false);
    clearSearchButton.setPreferredSize(new Dimension(24, 24));
    clearSearchButton.setMinimumSize(new Dimension(24, 24));
    toolBar.add(clearSearchButton);
    LayoutTools.constrains(toolPanel, toolBar, 0, 0, 1, 1,
                        GridBagConstraints.NONE, GridBagConstraints.WEST,
                        0.0, 0.0, 0, 0, 0, 0);

    LayoutTools.constrains(toolPanel, new JSeparator(JSeparator.VERTICAL), 1, 0, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
                        1.0, 0.0, 0, 0, 0, 0);

    toolBar = new JToolBar();
    toolBar.setRollover(true);
    icon = new ImageIcon(classLoader.getResource("images/about.png"));
    aboutButton = new JButton(icon);
    icon = new ImageIcon(classLoader.getResource("images/about-rollover.png"));
    aboutButton.setRolloverIcon(icon);
    aboutButton.setFocusPainted(false);
    aboutButton.addActionListener(this);
    aboutButton.setPreferredSize(new Dimension(24, 24));
    aboutButton.setMinimumSize(new Dimension(24, 24));
    aboutButton.setMaximumSize(new Dimension(24, 24));
    aboutButton.setToolTipText("A propos de Placide");
    toolBar.add(aboutButton);
    icon = new ImageIcon(classLoader.getResource("images/license.png"));
    licenseButton = new JButton(icon);
    icon = new ImageIcon(classLoader.getResource("images/license-rollover.png"));
    licenseButton.setRolloverIcon(icon);
    licenseButton.setFocusPainted(false);
    licenseButton.addActionListener(this);
    licenseButton.setPreferredSize(new Dimension(24, 24));
    licenseButton.setMinimumSize(new Dimension(24, 24));
    licenseButton.setMaximumSize(new Dimension(24, 24));
    licenseButton.setToolTipText("Licence");
    toolBar.add(licenseButton);

    LayoutTools.constrains(toolPanel, toolBar, 2, 0, 1, 1,
                        GridBagConstraints.NONE, GridBagConstraints.EAST,
                        0.0, 0.0, 0, 0, 0, 0);

    LayoutTools.constrains(this, toolPanel, 0, 1, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
                        1.0, 0.0, 0, 0, 0, 0);

    LayoutTools.constrains(this, new JSeparator(JSeparator.HORIZONTAL), 0, 2, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTH,
                        1.0, 0.0, 0, 0, 0, 0);

    searchPanel = new JPanel(new GridBagLayout());
    searchPanel.setMinimumSize(new Dimension(WIDTH, 65));
    label = new JLabel("Rechercher les communes dont le nom contient :");
    LayoutTools.constrains(searchPanel, label, 0, 0, 2, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                        0.0, 0.0, 0, 5, 0, 0);
    namePartTextField = new JTextField();
    namePartTextField.addActionListener(this);
    LayoutTools.constrains(searchPanel, namePartTextField, 0, 1, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                        1.0, 0.0, 0, 5, 5, 5);
    icon = new ImageIcon(classLoader.getResource("images/search.png"));
    searchButton = new JButton(icon);
    icon = new ImageIcon(classLoader.getResource("images/search-rollover.png"));
    searchButton.setRolloverIcon(icon);
    searchButton.setFocusPainted(false);
    searchButton.addActionListener(this);
    searchButton.setPreferredSize(new Dimension(24, 24));
    searchButton.setMinimumSize(new Dimension(24, 24));
    searchButton.setMaximumSize(new Dimension(24, 24));
    searchButton.setToolTipText("Rechercher");
    LayoutTools.constrains(searchPanel, searchButton, 1, 1, 1, 1,
                        GridBagConstraints.NONE, GridBagConstraints.EAST,
                        0.0, 0.0, 0, 0, 5, 5);
    b = BorderFactory.createEtchedBorder();
    tb = BorderFactory.createTitledBorder(b, "Recherche");
    tb.setTitleJustification(TitledBorder.LEFT);
    searchPanel.setBorder(tb);
    LayoutTools.constrains(this, searchPanel, 0, 3, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
                        1.0, 0.0, 5, 0, 0, 0);

    resultPanel = new JPanel(new GridBagLayout());
    resultPanel.setPreferredSize(new Dimension(WIDTH, 236));
    resultPanel.setMinimumSize(new Dimension(WIDTH, 236));
    resultList = new JList();
    resultList.getInputMap().setParent(inputMap);
    resultList.getActionMap().setParent(actionMap);
    resultList.setCellRenderer(new CommuneSearchResultListCellRenderer());
    resultList.addListSelectionListener(this);
    resultList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    resultList.setAutoscrolls(true);
    resultPane = new JScrollPane(resultList);
    resultPane.setMinimumSize(new Dimension(0, 100));

    LayoutTools.constrains(resultPanel, resultPane, 0, idx++, 2, 1,
                        GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
                        1.0, 1.0, 0, 5, 0, 5);
    label = new JLabel("Nombre de rsultats : ");
    LayoutTools.constrains(resultPanel, label, 0, idx, 1, 1,
                        GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
                        0.0, 0.0, 0, 5, 0, 0);
    resultNumberLabel = new JLabel();
    resultNumberLabel.setHorizontalAlignment(resultNumberLabel.LEFT);
    LayoutTools.constrains(resultPanel, resultNumberLabel, 1, idx, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
                        1.0, 0.0, 0, 0, 0, 5);
    b = BorderFactory.createEtchedBorder();
    tb = BorderFactory.createTitledBorder(b, "Rsultats");
    tb.setTitleJustification(TitledBorder.LEFT);
    resultPanel.setBorder(tb);
    LayoutTools.constrains(this, resultPanel, 0, 4, 1, 1,
                        GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
                        1.0, 1.0, 0, 0, 0, 0);

    detailsIcons = new ImageIcon[7];
    detailsIcons[0] = new ImageIcon(classLoader.getResource("images/commune_old_name.png"));
    detailsIcons[1] = new ImageIcon(classLoader.getResource("images/commune.png"));
    detailsIcons[2] = new ImageIcon(classLoader.getResource("images/commune_association.png"));
    detailsIcons[3] = new ImageIcon(classLoader.getResource("images/commune_old.png"));
    detailsIcons[4] = new ImageIcon(classLoader.getResource("images/commune_old_code.png"));
/*
    detailsIcons[5] = new ImageIcon(classLoader.getResource("images/5.gif"));
    detailsIcons[6] = new ImageIcon(classLoader.getResource("images/6.gif"));
*/

    detailPanel = new JPanel(new GridBagLayout());
    detailPanel.setPreferredSize(new Dimension(WIDTH, 170));
    detailPanel.setMinimumSize(new Dimension(WIDTH, 170));
    detailPanel.setMaximumSize(new Dimension(WIDTH, 170));
    b = BorderFactory.createEtchedBorder();
    tb = BorderFactory.createTitledBorder(b, "Dtails");
    tb.setTitleJustification(TitledBorder.LEFT);
    detailPanel.setBorder(tb);

    topDetailPanel = new JPanel(new GridBagLayout());
    details = new JLabel[10];
    details[9] = new JLabel(" ");
    details[9].setPreferredSize(new Dimension(48, 22));
    details[9].setMaximumSize(new Dimension(48, 22));
    LayoutTools.constrains(topDetailPanel, details[9], 0, 0, 1, 1,
                        GridBagConstraints.NONE, GridBagConstraints.WEST,
                        0.0, 0.0, 0, 0, 0, 0);
    details[0] = new JLabel();
    LayoutTools.constrains(topDetailPanel, details[0], 1, 0, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.SOUTH,
                        1.0, 0.0, 0, 5, 0, 0);
    icon = new ImageIcon(classLoader.getResource("images/commune_history.png"));
    historyButton = new JButton(icon);
    icon = new ImageIcon(classLoader.getResource("images/commune_history-rollover.png"));
    historyButton.setRolloverIcon(icon);
    icon = new ImageIcon(classLoader.getResource("images/commune_history-disabled.png"));
    historyButton.setDisabledIcon(icon);
    historyButton.addActionListener(this);
    historyButton.setFocusPainted(false);
    historyButton.setEnabled(false);
    historyButton.setPreferredSize(new Dimension(24, 24));
    historyButton.setMaximumSize(new Dimension(24, 24));
    historyButton.setToolTipText("Voir l'historique des modifications (Ctrl+H)");
    LayoutTools.constrains(topDetailPanel, historyButton, 2, 0, 2, 1,
                        GridBagConstraints.NONE, GridBagConstraints.EAST,
                        0.0, 0.0, 0, 0, 0, 0);
    LayoutTools.constrains(detailPanel, topDetailPanel, 0, 0, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                        1.0, 0.0, 0, 5, 0, 5);

    bottomDetailPanel = new JPanel(new GridBagLayout());
    for(int i=1; i<details.length-1; i++) {
      details[i] = new JLabel(" ");
      LayoutTools.constrains(bottomDetailPanel, details[i], 0, i-1, 3, 1,
                          GridBagConstraints.NONE, GridBagConstraints.WEST,
                          0.0, 0.0, 0, 0, 0, 0);
    }
    LayoutTools.constrains(detailPanel, bottomDetailPanel, 0, 1, 1, 1,
                        GridBagConstraints.NONE, GridBagConstraints.WEST,
                        0.0, 0.0, 5, 5, 0, 5);

    LayoutTools.constrains(this, detailPanel, 0, 5, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                        1.0, 0.0, 0, 0, 0, 0);
    resultList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_H, InputEvent.CTRL_MASK), "showhistory");
    resultList.getActionMap().put("showhistory", new HitKeyAction (historyButton));
    resultList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, InputEvent.CTRL_MASK - InputEvent.CTRL_MASK), "up");
    resultList.getActionMap().put("up", new UpAction (resultList));
    resultList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, InputEvent.CTRL_MASK - InputEvent.CTRL_MASK), "down");
    resultList.getActionMap().put("down", new DownAction (resultList));
    resultList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_UP, InputEvent.CTRL_MASK - InputEvent.CTRL_MASK), "pageup");
    resultList.getActionMap().put("pageup", new PageUpAction (resultList));
    resultList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_DOWN, InputEvent.CTRL_MASK - InputEvent.CTRL_MASK), "pagedown");
    resultList.getActionMap().put("pagedown", new PageDownAction (resultList));
    resultList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, InputEvent.CTRL_MASK - InputEvent.CTRL_MASK), "previous");
    resultList.getActionMap().put("previous", new HitKeyAction (previousButton.getButton()));
    resultList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, InputEvent.CTRL_MASK - InputEvent.CTRL_MASK), "next");
    resultList.getActionMap().put("next", new HitKeyAction (nextButton.getButton()));

    BevelBorder bb = (BevelBorder) BorderFactory.createLoweredBevelBorder();
    HistoryDialog.setHighlightOuterColor(bb.getHighlightOuterColor(resultPane));
    HistoryDialog.setHighlightInnerColor(getBackground());
    HistoryDialog.setShadowOuterColor(bb.getShadowOuterColor(resultPane));
    HistoryDialog.setShadowInnerColor(bb.getShadowInnerColor(resultPane));
  }

  public void setOwner(JFrame owner) {
    this.owner = owner;
  }

  public void activate() {
    namePartTextField.requestFocus();
  }

  public void activateResult() {
    resultList.requestFocus();
  }

  public void setNewSearchAction(Action action) {
    ImageIcon icon = null;

    newSearchButton.setAction(action);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/search_new.png"));
    newSearchButton.setIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/search_new-rollover.png"));
    newSearchButton.setRolloverIcon(icon);
    inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK), "newsearch");
    actionMap.put("newsearch", action);
    newSearchButton.setToolTipText("Nouvelle recherche (Ctrl+N)");
  }

  public void setSaveAction(Action action) {
    ImageIcon icon = null;

    saveButton.setAction(action);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/save.png"));
    saveButton.setIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/save-rollover.png"));
    saveButton.setRolloverIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/save-disabled.png"));
    saveButton.setDisabledIcon(icon);
    inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK), "save");
    actionMap.put("save", action);
    saveButton.setToolTipText("Enregistrer les rsultats (Ctrl+S)");
  }

  public void setCopyAction(Action action) {
    ImageIcon icon = null;

    copyButton.setAction(action);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/copy.png"));
    copyButton.setIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/copy-rollover.png"));
    copyButton.setRolloverIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/copy-disabled.png"));
    copyButton.setDisabledIcon(icon);
    resultList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK), "copy");
    resultList.getActionMap().put("copy", action);
    copyButton.setToolTipText("Copier la slection (Ctrl+C)");
  }

  public void setDeleteSearchAction(Action action) {
    ImageIcon icon = null;

    deleteSearchButton.setAction(action);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/results_delete.png"));
    deleteSearchButton.setIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/results_delete-rollover.png"));
    deleteSearchButton.setRolloverIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/results_delete-disabled.png"));
    deleteSearchButton.setDisabledIcon(icon);
    inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, InputEvent.CTRL_MASK-InputEvent.CTRL_MASK), "delete");
    actionMap.put("delete", action);
    deleteSearchButton.setToolTipText("Supprimer les rsultats affichs (Suppr)");
  }

  public void setClearSearchAction(Action action) {
    ImageIcon icon = null;

    clearSearchButton.setAction(action);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/results_clear.png"));
    clearSearchButton.setIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/results_clear-rollover.png"));
    clearSearchButton.setRolloverIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/results_clear-disabled.png"));
    clearSearchButton.setDisabledIcon(icon);
    clearSearchButton.setToolTipText("Supprimer tous les rsultats (Ctrl+Suppr)");
    inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, InputEvent.CTRL_MASK), "clear");
    actionMap.put("clear", action);
  }

  public void setPreviousAction(Action action) {
    ImageIcon icon = null;

    previousButton.setDefaultAction(action);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/previous.png"));
    previousButton.setIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/previous-rollover.png"));
    previousButton.setRolloverIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/previous-disabled.png"));
    previousButton.setDisabledIcon(icon);
    previousButton.setToolTipText("Rsultats prcdents (<-)");
  }

  public void insertPreviousAction(Action action, int i) {
    previousButton.insertAction(action, i);
  }

  public Action removePreviousAction(int i) {
    return previousButton.removeAction(i);
  }

  public void setNextAction(Action action) {
    ImageIcon icon = null;

    nextButton.setDefaultAction(action);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/next.png"));
    nextButton.setIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/next-rollover.png"));
    nextButton.setRolloverIcon(icon);
    icon = new ImageIcon(getClass().getClassLoader().getResource("images/next-disabled.png"));
    nextButton.setDisabledIcon(icon);
    nextButton.setToolTipText("Rsultats suivants (->)");
  }

  public void insertNextAction(Action action, int i) {
    nextButton.insertAction(action, i);
  }

  public Action removeNextAction(int i) {
    return nextButton.removeAction(i);
  }

  public void setPreviousEnabled(boolean enabled) {
    previousButton.setEnabled(enabled);
  }

  public void setNextEnabled(boolean enabled) {
    nextButton.setEnabled(enabled);
  }

  public void setNamePart(String namePart) {
    namePartTextField.setText(namePart);
  }

  public void setResults(String[][] results, int selected) {
    resultList.setListData(results);
    resultList.setSelectedIndex(selected);
    resultList.ensureIndexIsVisible(selected);
    if(results.length > 0) {
      resultList.requestFocus();
    } else {
      namePartTextField.requestFocus();
      namePartTextField.selectAll();
    }
    resultNumberLabel.setText(""+results.length);
  }

  public void setDetails(String[] detail, short iconIndex, boolean modified) {
    if(iconIndex == -1) {
      details[9].setIcon(null);
    } else {
      details[9].setIcon(detailsIcons[iconIndex]);
    }
    for(int i=0; i<detail.length; i++) {
      details[i].setText(detail[i]);
    }
    historyButton.setEnabled(modified);
  }

  public void addCommuneSearchPanelListener(CommuneSearchPanelListener listener) {
    synchronized (listeners) {
      if(!listeners.contains(listener)) {
        listeners.addElement(listener);
      }
    }
  }

  public void removeCommuneSearchPanelListener(CommuneSearchPanelListener listener) {
    synchronized (listeners) {
      if(listeners.contains(listener)) {
        listeners.removeElement(listener);
      }
    }
  }

  public void actionPerformed(ActionEvent event) {
    CommuneSearchPanelListener l = null;
    Object                     s = null;
    JDialog                    d = null;

    s = event.getSource();
    if((s == namePartTextField) || (s == searchButton)) {
      synchronized (listeners) {
        for(int i=0; i<listeners.size(); i++) {
          l = (CommuneSearchPanelListener)listeners.elementAt(i);
          l.searchRequested(namePartTextField.getText());
        }
      }
    } else if(s == historyButton) {
      synchronized (listeners) {
        for(int i=0; i<listeners.size(); i++) {
          l = (CommuneSearchPanelListener)listeners.elementAt(i);
          l.historyRequested();
        }
      }
    } else if(s == aboutButton) {
        d = new AboutDialog(owner);
        d.setVisible(true);
    } else if(s == licenseButton) {
        d = new LicenseDialog(owner);
        d.setVisible(true);
    }
  }

  public void valueChanged(ListSelectionEvent e) {
    CommuneSearchPanelListener cspl = null;

    synchronized (listeners) {
      for(int i=0; i<listeners.size(); i++) {
        cspl = (CommuneSearchPanelListener)listeners.elementAt(i);
        cspl.resultSelectionChanged(resultList.getSelectedIndex());
      }
    }
  }
}