/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.placide.widgets;

import java.awt.Dimension;
import java.awt.Component;

import javax.swing.UIManager;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;


public class CommuneSearchResultListCellRenderer extends JTable implements ListCellRenderer {
  private Border noFocusBorder = null;

  public CommuneSearchResultListCellRenderer() {
    DefaultTableCellRenderer dtcr = null;

    if (noFocusBorder == null) {
      noFocusBorder = new EmptyBorder(1, 1, 1, 1);
    }

    setModel(new DefaultTableModel(1, 3));
    dtcr = new DefaultTableCellRenderer();
    dtcr.setHorizontalAlignment(dtcr.RIGHT);
    getColumnModel().getColumn(0).setCellRenderer(dtcr);
    getColumnModel().getColumn(2).setCellRenderer(dtcr);
    setOpaque(true);
    setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
    getColumnModel().getColumn(0).setPreferredWidth(22);
    getColumnModel().getColumn(0).setMaxWidth(44);
    getColumnModel().getColumn(1).setPreferredWidth(266);
    getColumnModel().getColumn(2).setPreferredWidth(20);
    getColumnModel().getColumn(2).setMaxWidth(40);
    setShowGrid(false);
    setIntercellSpacing(new Dimension(0,0));
  }

  public Component getListCellRendererComponent(JList list,
                                                Object value,
                                                int index,
                                                boolean isSelected,
                                                boolean cellHasFocus) {
    String[] values   = null;

    if(isSelected) {
      setForeground(list.getSelectionForeground());
      setBackground(list.getSelectionBackground());
    } else {
      setForeground(list.getForeground());
      setBackground(list.getBackground());
    }
    values = (String[]) value;
    getModel().setValueAt(values[0], 0, 0);
    getModel().setValueAt(values[1], 0, 1);
    getModel().setValueAt(values[2], 0, 2);

    setEnabled(list.isEnabled());
    setFont(list.getFont());
    setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);

    return this;
  }
}