/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.placide.widgets;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import java.util.Vector;
import java.text.SimpleDateFormat;

import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.border.BevelBorder;

import net.jourand.olaf.tools.LayoutTools;

public class HistoryDialog extends JDialog implements ActionListener {
  private static Color HIGHLIGHTOUTERCOLOR = null;
  private static Color HIGHLIGHTINNERCOLOR = null;
  private static Color SHADOWOUTERCOLOR = null;
  private static Color SHADOWINNERCOLOR = null;

  private int              line   = 0;
  private SimpleDateFormat sdf    = null;
  private Font             bFont  = null;
  private Font             sFont  = null;
  private JPanel           panel  = null;
  private JButton          button = null;

  public HistoryDialog(Frame owner, String title) {
    super(owner, title, true);

    BevelBorder border = null;

    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    setResizable(false);
    getContentPane().setLayout(new GridBagLayout());
    sdf = new SimpleDateFormat("dd/MM/yyyy");
    bFont = getFont().deriveFont(Font.BOLD);
    sFont = getFont().deriveFont((float)(getFont().getSize2D()-1.0));
    panel = new JPanel(new GridBagLayout());
    border = (BevelBorder) BorderFactory.createLoweredBevelBorder();
    panel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED,
      HIGHLIGHTOUTERCOLOR,
      HIGHLIGHTINNERCOLOR,
      SHADOWOUTERCOLOR,
      SHADOWINNERCOLOR));
    panel.setBackground(Color.white);
    LayoutTools.constrains(getContentPane(), panel, 0, 0, 1, 1,
                        GridBagConstraints.NONE, GridBagConstraints.CENTER,
                        0.0, 0.0, 5, 5, 5, 5);
    button = new JButton("Fermer");
    button.setSelected(true);
    button.setFocusPainted(false);
    button.addActionListener(this);
    LayoutTools.constrains(getContentPane(), button, 0, 1, 1, 1,
                        GridBagConstraints.NONE, GridBagConstraints.CENTER,
                        0.0, 0.0, 5, 0, 10, 0);
  }

  static void setHighlightOuterColor(Color highlightOuterColor) {
    HIGHLIGHTOUTERCOLOR = highlightOuterColor;
  }

  static void setHighlightInnerColor(Color highlightInnerColor) {
    HIGHLIGHTINNERCOLOR = highlightInnerColor;
  }

  static void setShadowOuterColor(Color shadowOuterColor) {
    SHADOWOUTERCOLOR = shadowOuterColor;
  }

  static void setShadowInnerColor(Color shadowInnerColor) {
    SHADOWINNERCOLOR = shadowInnerColor;
  }

  public void add(Date date, String title, String legal, String jo, String effective, Vector description) {
    JLabel label = null;
    int    top   = 0;

    if(date != null) {
      label = new JLabel(sdf.format(date));
    } else {
      label = new JLabel("");
    }
    label.setFont(bFont);
    if(line != 0) {
      top = 5;
    }
    LayoutTools.constrains(panel, label, 0, line, 1, 1,
                        GridBagConstraints.NONE, GridBagConstraints.WEST,
                        0.0, 0.0, top, 5, 0, 0);
    if(title != null) {
      label = new JLabel(title);
    } else {
      label = new JLabel("");
    }
    label.setFont(bFont);
    LayoutTools.constrains(panel, label, 1, line++, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                        0.0, 0.0, top, 5, 0, 5);
    top = 0;
    for(int i=0; i<description.size(); i++) {
      label = new JLabel((String)description.elementAt(i));
      LayoutTools.constrains(panel, label, 1, line++, 1, 1,
                          GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                          0.0, 0.0, 0, 5, 0, 5);
    }
    if(legal != null) {
      label = new JLabel(legal);
    } else {
      label = new JLabel("");
    }
    label.setFont(sFont);
    LayoutTools.constrains(panel, label, 1, line++, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                        0.0, 0.0, 0, 5, 0, 5);
    if(jo != null) {
      label = new JLabel(jo);
    } else {
      label = new JLabel("");
    }
    label.setFont(sFont);
    LayoutTools.constrains(panel, label, 1, line++, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                        0.0, 0.0, 0, 5, 0, 5);
    if(effective != null) {
      label = new JLabel(effective);
    } else {
      label = new JLabel("");
    }
    label.setFont(sFont);
    LayoutTools.constrains(panel, label, 1, line++, 1, 1,
                        GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                        0.0, 0.0, 0, 5, 0, 5);
  }

  public void actionPerformed(ActionEvent e) {
    dispose();
  }
}
