/******************************************************************************
 *                                                                            *
 * This work is licensed under the Creative Commons                           *
 * Attribution-NoDerivs-NonCommercial License. To view a copy of this         *
 * license, visit http://creativecommons.org/licenses/by-nd-nc/1.0/ or send a *
 * letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California    *
 * 94305, USA.                                                                *
 *                                                                            *
 *                                                                            *
 *                               (cc) creative                                *
 *                                     commons                                *
 *                                COMMONS DEED                                *
 *                                                                            *
 *                  Attribution-NonCommercial-ShareAlike 1.0                  *
 *                                                                            *
 * You are free:                                                              *
 *   -to copy, distribute, display, and perform the work                     *
 *   - to make derivative works                                               *
 *                                                                            *
 * Under the following conditions:                                            *
 *   BY: Attribution. You must give the original author credit.               *
 *   NC: Noncommercial. You may not use this work for commercial purposes.    *
 *   SA: Share Alike. If you alter, transform, or build upon this work, you   *
 *       may distribute the resulting work only under a license identical to  *
 *       this one.                                                            *
 *                                                                            *
 *   -For any reuse or distribution, you must make clear to others the       *
 *     license terms of this work.                                            *
 *   -Any of these conditions can be waived if you get permission from the   *
 *     author.                                                                *
 *                                                                            *
 * Your fair use and other rights are in no way affected by the above.        *
 *                                                                            *
 * This is a human-readable summary of the Legal Code (the full license)      *
 * available on http://creativecommons.org/licenses/by-nc-sa/1.0/legalcode    *
 *                                                                            *
 *                                                                            *
 * Disclaimer:                                                                *
 *                                                                            *
 *   The Commons Deed is not a license. It is simply a handy reference for    *
 *   understanding the Legal Code (the full license) - it is a human-readable *
 *   expression of some of its key terms. Think of it as the user-friendly    *
 *   interface to the Legal Code beneath. This Deed itself has no legal       *
 *   value, and its contents do not appear in the actual license.             *
 *                                                                            *
 *   Creative Commons is not a law firm and does not provide legal services.  *
 *   Distributing of, displaying of, or linking to this Commons Deed does not *
 *   create an attorney-client relationship.                                  *
 *                                                                            *
 ******************************************************************************/
package net.jourand.placide.widgets;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JSeparator;
import javax.swing.ImageIcon;
import javax.swing.text.html.HTMLDocument;

import net.jourand.olaf.tools.LayoutTools;

public class LicenseDialog  extends JDialog implements ActionListener {
    private JButton   button = null;

    public LicenseDialog(Frame owner) {
        super(owner, "Licence", true);

        JTextPane    pane   = null;
        JScrollPane  sPane  = null;
        JPanel       panel  = null;
        JTabbedPane  tPane  = null;
        URL          url    = null;
        HTMLDocument html   = null;
        ClassLoader  cl     = null;
        ImageIcon    icon   = null;
        JLabel       banner = null;

        cl = getClass().getClassLoader();

        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        setSize(new Dimension(510, 388));
        setResizable(false);
        getContentPane().setLayout(new GridBagLayout());

        icon = new ImageIcon(cl.getResource("images/placide-banner.png"));
        banner = new JLabel(icon);
        LayoutTools.constrains(this.getContentPane(), banner, 0, 0, 1, 1,
                            GridBagConstraints.NONE, GridBagConstraints.CENTER,
                            0.0, 0.0, 0, 0, 0, 0);
        LayoutTools.constrains(this.getContentPane(), new JSeparator(JSeparator.HORIZONTAL), 0, 1, 1, 1,
                            GridBagConstraints.BOTH, GridBagConstraints.NORTH,
                            1.0, 1.0, 0, 0, 0, 0);

        tPane = new JTabbedPane();
        tPane.setPreferredSize(new Dimension(200, 260));
        tPane.setMinimumSize(new Dimension(200, 260));

        url = cl.getResource("doc/licenses/license.html");
        html = new HTMLDocument();
        pane = new JTextPane(html);
        try {
            pane.setPage(url);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
        pane.setEditable(false);
        sPane = new JScrollPane(pane);
        panel = new JPanel(new GridBagLayout());
        panel.setMinimumSize(new Dimension(100, 205));
        LayoutTools.constrains(panel, sPane, 0, 0, 1, 1,
                            GridBagConstraints.BOTH, GridBagConstraints.CENTER,
                            1.0, 1.0, 10, 10, 10, 10);
        tPane.addTab("Deed", panel);

        url = cl.getResource("doc/licenses/cc.html");
        html = new HTMLDocument();
        pane = new JTextPane(html);
        try {
            pane.setPage(url);
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
        pane.setEditable(false);
        sPane = new JScrollPane(pane);
        panel = new JPanel(new GridBagLayout());
        LayoutTools.constrains(panel, sPane, 0, 0, 1, 1,
                            GridBagConstraints.BOTH, GridBagConstraints.CENTER,
                            1.0, 1.0, 10, 10, 10, 10);
        tPane.addTab("Legal Code", panel);

        LayoutTools.constrains(this.getContentPane(), tPane, 0, 2, 1, 1,
                            GridBagConstraints.BOTH, GridBagConstraints.CENTER,
                            1.0, 1.0, 10, 10, 10, 10);

        button = new JButton("Fermer");
        button.setFocusPainted(false);
        button.addActionListener(this);
        LayoutTools.constrains(this.getContentPane(), button, 0, 3, 1, 1,
                            GridBagConstraints.NONE, GridBagConstraints.CENTER,
                            0.0, 0.0, 0, 0, 10, 10);
        LayoutTools.centerOnWindow(this, owner);

    }

    public void actionPerformed(ActionEvent e) {
        dispose();
    }

    public void paint(Graphics g) {
        super.paint(g);
        button.setSelected(true);
        button.requestFocus();
    }

    public static void main(String[] args) {
        LicenseDialog af = null;

        af = new LicenseDialog(new Frame());
        af.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        af.setSize(new Dimension(400, 410));
//        af.pack();
        af.setVisible(true);
    }
}
